package PZ::Helpers::Core;

use base 'Mojolicious::Plugin';

use YAML;
use Mojo::URL;

use constant TRACKERS => [qw(
    fbclid
    gclid
    utm_medium
    utm_source
    utm_campaign
    utm_content
    utm_term
)];

sub register  {
    my ($class, $self ) = @_;

    $self->helper(error => sub {
        my $c      = shift;
        my $status = shift;
        my $errors = [];

        if ( scalar @_ == 2 ) {
            $errors = [{ code => shift, message => shift }];
        }
        elsif ( ref $_[0] eq 'ARRAY' ) {
            $errors = shift;
        }
        elsif ( ref $_[0] eq 'HASH' ) {
            $errors = [ shift ];
        }
        else {
            $errors = [{ message => shift, code => undef }];
        }

        $c->stash(
            status  => $status,
            openapi => { errors => $errors }
        );

        return undef;
    });

    $self->helper( trace => sub {
        my $c      = shift;
        my $data   = shift // '';

        $data = Dump $data if ref $data;
        $c->app->log->debug($data);

    });

    $self->helper( spec_filter => sub {
        my $c     = shift;
        my $data  = shift;
        my $class = shift;

        if (my $def =$c->openapi->spec("/components/schemas/$class")) {
            my $filtered = {};

            KEY:
            foreach my $key ( keys %{ $def->{properties} } ) {
                my $value    = $data->{$key};
                my $nullable = 0;

                my $types = $def->{properties}{$key}{type};

                if ( ref $types eq 'ARRAY' ) {
                    TYPE:
                    foreach my $type ( @{ $types } ) {
                        $nullable = 1, last if $type eq 'null';
                    }
                }

                if ( $key =~ /^cirpack_/ && $value eq 'XXX' ) {
                    $value  = $nullable ? undef : '';
                }

                $filtered->{$key} = $value;
            }

            $data = $filtered;

        }

        return $data;
    });

    $self->helper( shortcut => sub {
        my $c  = shift;
        my $id = shift // $c->stash->{id};
        my $user = $c->stash->{user} || $c->current_user;
        my $shortcut = $c->schema->resultset('Shortcut')->find({ id => $id });
        return $c->error(404, 'NOT_FOUND') if ! $shortcut;
        return $c->error(403, 'ACCESS_DENIED') if $shortcut->user_id != $c->user->id;
        return $shortcut;
    });

    $self->helper( trackers => sub {
        return @{ TRACKERS() };
    });

    $self->helper( sanitize_url => sub {
        my $c = shift;

        my $url = Mojo::URL->new(shift);

        foreach my $tracker ( $c->trackers() ) {
            $url->query({ $tracker => undef });
        }
        return $url->to_string;
    });

    $self->helper( user => sub {
        my $c      = shift;
        return $c->stash->{user};
    });
}

1;

