-- 1 up
create sequence "uid_seq" start 100000;

create table "users" (
    "id" integer not null default nextval('uid_seq'),
    "uuid" varchar(36) not null,
    "created" timestamp(0) not null default now(),
    "deleted" timestamp(0),
    "is_active" boolean not null default true,
    "username" text,
    "displayname" text,
    "token" text,
    primary key("id"),
    unique("uuid"),
    unique("token")
);

create table "shortcuts" (
    "id" integer not null default nextval('uid_seq'),
    "user_id" integer not null,
    "created" timestamp(0) not null default now(),
    "deleted" timestamp(0),
    "is_active" boolean not null default true,
    "shortcut" varchar(8) not null,
    "url" text,
    "code" integer not null default 301,
    "title" text,
    "description" text,
    primary key("id"),
    foreign key("user_id") references "users" ("id") on update cascade on delete restrict,
    unique("shortcut")
);

create table "log" (
    "id" integer not null default nextval('uid_seq'),
    "time" timestamp(0) not null default now(),
    "shortcut_id" integer not null,
    "ip" inet,
    primary key("id"),
    foreign key("shortcut_id") references "shortcuts" ("id") on update cascade on delete restrict
);

-- 2 up
alter table "shortcuts" alter column "code" set default 302;
alter table "shortcuts" drop constraint "shortcuts_shortcut_key";
create index "shortcuts_shortcut_idx" on "shortcuts" ("shortcut");

-- 3 up
alter table "log" add column "ua" text;

-- 4 up
alter table "log" add column "referrer" text;

-- 5 up
create view "stat_hourly" as
select
    "shortcut_id",
    date_trunc('hour', "time") as "period",
    count(*)
from "log"
    group by "shortcut_id", date_trunc('hour', "time")
    order by date_trunc('hour', "time")
;

create view "stat_daily" as
select
    "shortcut_id",
    date_trunc('day', "time") as "period",
    count(*)
from "log"
    group by "shortcut_id", date_trunc('day', "time")
    order by date_trunc('day', "time")
;

-- 6 up
alter table "shortcuts" add column "counter" integer not null default 0;

-- 7 up
alter table "shortcuts" alter column "shortcut" type varchar(32);

